@@ -13,6 +13,7 @@ updating the file tree explorer associated with a project
 
 # Standard library imports
 import configparser
+import os
 import os.path as osp
 import shutil
 import functools
@@ -21,46 +22,62 @@ from collections import OrderedDict
 # Third party imports
 from qtpy.compat import getexistingdirectory
 from qtpy.QtCore import Signal, Slot
-from qtpy.QtWidgets import QInputDialog, QMenu, QMessageBox, QVBoxLayout
+from qtpy.QtWidgets import QInputDialog, QMessageBox
 
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
+from spyder.api.plugin_registration.decorators import on_plugin_available
 from spyder.api.translations import get_translation
-from spyder.api.plugins import Plugins, SpyderPluginWidget
+from spyder.api.plugins import Plugins, SpyderDockablePlugin
 from spyder.config.base import (get_home_dir, get_project_config_folder,
                                 running_under_pytest)
-from spyder.config.manager import CONF
 from spyder.py3compat import is_text_string, to_text_string
 from spyder.utils import encoding
 from spyder.utils.icon_manager import ima
-from spyder.utils.qthelpers import add_actions, create_action, MENU_SEPARATOR
 from spyder.utils.misc import getcwd_or_home
+from spyder.plugins.mainmenu.api import ApplicationMenus, ProjectsMenuSections
 from spyder.plugins.projects.api import (BaseProjectType, EmptyProject,
                                          WORKSPACE)
 from spyder.plugins.projects.utils.watcher import WorkspaceWatcher
-from spyder.plugins.projects.widgets.explorer import ProjectExplorerWidget
+from spyder.plugins.projects.widgets.main_widget import ProjectExplorerWidget
 from spyder.plugins.projects.widgets.projectdialog import ProjectDialog
 from spyder.plugins.completion.api import (
     CompletionRequestTypes, FileChangeType, WorkspaceUpdateKind)
 from spyder.plugins.completion.decorators import (
     request, handles, class_register)
 
-
 # Localization
 _ = get_translation("spyder")
 
 
+class ProjectsMenuSubmenus:
+    RecentProjects = 'recent_projects'
+
+
+class ProjectsActions:
+    NewProject = 'new_project_action'
+    OpenProject = 'open_project_action'
+    CloseProject = 'close_project_action'
+    DeleteProject = 'delete_project_action'
+    ClearRecentProjects = 'clear_recent_projects_action'
+    MaxRecent = 'max_recent_action'
+
+
+class RecentProjectsMenuSections:
+    Recent = 'recent_section'
+    Extras = 'extras_section'
+
+
 @class_register
-class Projects(SpyderPluginWidget):
+class Projects(SpyderDockablePlugin):
     """Projects plugin."""
-
-    CONF_SECTION = 'project_explorer'
-    CONF_FILE = False
-
-    # This is required for the new API
     NAME = 'project_explorer'
+    CONF_SECTION = NAME
+    CONF_FILE = False
     REQUIRES = []
-    OPTIONAL = [Plugins.Completions, Plugins.IPythonConsole, Plugins.Explorer]
+    OPTIONAL = [Plugins.Completions, Plugins.IPythonConsole, Plugins.Editor,
+                Plugins.MainMenu]
+    WIDGET_CLASS = ProjectExplorerWidget
 
     # Signals
     sig_project_created = Signal(str, str, object)
@@ -79,236 +96,241 @@ class Projects(SpyderPluginWidget):
     """
 
     sig_project_loaded = Signal(object)
-    sig_project_closed = Signal(object)
+    """
+    This signal is emitted when a project is loaded.
+
+    Parameters
+    ----------
+    project_path: object
+        Loaded project path.
+    """
+
+    sig_project_closed = Signal((object,), (bool,))
+    """
+    This signal is emitted when a project is closed.
+
+    Parameters
+    ----------
+    project_path: object
+        Closed project path (signature 1).
+    close_project: bool
+        This is emitted only when closing a project but not when switching
+        between projects (signature 2).
+    """
+
     sig_pythonpath_changed = Signal()
+    """
+    This signal is emitted when the Python path has changed.
+    """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, configuration=None):
         """Initialization."""
-        SpyderPluginWidget.__init__(self, parent)
-
-        self.explorer = ProjectExplorerWidget(
-            self,
-            name_filters=self.get_option('name_filters'),
-            show_hscrollbar=self.get_option('show_hscrollbar'),
-            options_button=self.options_button,
-            single_click_to_open=CONF.get('explorer', 'single_click_to_open'),
-        )
-
-        layout = QVBoxLayout()
-        layout.addWidget(self.explorer)
-        self.setLayout(layout)
-
-        self.recent_projects = self.get_option('recent_projects', default=[])
+        super().__init__(parent, configuration)
+        self.recent_projects = self.get_conf('recent_projects', [])
         self.current_active_project = None
         self.latest_project = None
         self.watcher = WorkspaceWatcher(self)
         self.completions_available = False
-        self.explorer.setup_project(self.get_active_project_path())
+        self.get_widget().setup_project(self.get_active_project_path())
         self.watcher.connect_signals(self)
         self._project_types = OrderedDict()
 
-    #------ SpyderPluginWidget API ---------------------------------------------
-    def get_plugin_title(self):
-        """Return widget title"""
+    # ---- SpyderDockablePlugin API
+    # ------------------------------------------------------------------------
+    def get_name(self):
         return _("Project")
 
-    def get_focus_widget(self):
-        """
-        Return the widget to give focus to when
-        this plugin's dockwidget is raised on top-level
-        """
-        return self.explorer.treewidget
+    def get_description(self):
+        return _("Create Spyder projects and manage their files.")
 
-    def get_plugin_actions(self):
-        """Return a list of actions related to plugin"""
-        self.new_project_action = create_action(self,
-                                    _("New Project..."),
-                                    triggered=self.create_new_project)
-        self.open_project_action = create_action(self,
-                                    _("Open Project..."),
-                                    triggered=lambda v: self.open_project())
-        self.close_project_action = create_action(self,
-                                    _("Close Project"),
-                                    triggered=self.close_project)
-        self.delete_project_action = create_action(self,
-                                    _("Delete Project"),
-                                    triggered=self.delete_project)
-        self.clear_recent_projects_action = create_action(
-            self,
-            _("Clear this list"),
-            triggered=self.clear_recent_projects)
-        self.recent_project_menu = QMenu(_("Recent Projects"), self)
+    def get_icon(self):
+        return self.create_icon('project')
 
-        self.max_recent_action = create_action(
-            self,
-            _("Maximum number of recent projects..."),
-            triggered=self.change_max_recent_projects)
-
-        if self.main is not None:
-            self.main.projects_menu_actions += [self.new_project_action,
-                                                MENU_SEPARATOR,
-                                                self.open_project_action,
-                                                self.close_project_action,
-                                                self.delete_project_action,
-                                                MENU_SEPARATOR,
-                                                self.recent_project_menu,
-                                                self._toggle_view_action]
-
-        self.setup_menu_actions()
-        return []
-
-    def register_plugin(self):
+    def on_initialize(self):
         """Register plugin in Spyder's main window"""
-        ipyconsole = self.main.ipyconsole
-        treewidget = self.explorer.treewidget
-        completions = self.main.get_plugin(Plugins.Completions, error=False)
-        outlineexplorer = self.main.get_plugin(
-            Plugins.OutlineExplorer, error=False)
+        widget = self.get_widget()
+        treewidget = widget.treewidget
 
-        self.add_dockwidget()
-        self.explorer.sig_open_file_requested.connect(self.main.open_file)
+        self.ipyconsole = None
+        self.editor = None
+        self.completions = None
 
         treewidget.sig_delete_project.connect(self.delete_project)
-        treewidget.sig_open_file_requested.connect(self.main.editor.load)
-        treewidget.sig_removed.connect(self.main.editor.removed)
-        treewidget.sig_tree_removed.connect(self.main.editor.removed_tree)
-        treewidget.sig_renamed.connect(self.main.editor.renamed)
-        treewidget.sig_tree_renamed.connect(self.main.editor.renamed_tree)
-        treewidget.sig_module_created.connect(self.main.editor.new)
-        treewidget.sig_file_created.connect(
-            lambda t: self.main.editor.new(text=t))
-        treewidget.sig_open_interpreter_requested.connect(
-            ipyconsole.create_client_from_path)
         treewidget.sig_redirect_stdio_requested.connect(
-            self.main.redirect_internalshell_stdio)
-        treewidget.sig_run_requested.connect(
-            lambda fname:
-            ipyconsole.run_script(fname, osp.dirname(fname), '', False, False,
-                                  False, True, False))
+            self.sig_redirect_stdio_requested)
+        self.sig_switch_to_plugin_requested.connect(
+            lambda plugin, check: self.show_explorer())
+        self.sig_project_loaded.connect(self.update_explorer)
+
+        if self.main:
+            widget.sig_open_file_requested.connect(self.main.open_file)
+            self.main.project_path = self.get_pythonpath(at_start=True)
+            self.sig_project_loaded.connect(
+                lambda v: self.main.set_window_title())
+            self.sig_project_closed.connect(
+                lambda v: self.main.set_window_title())
+            self.main.restore_scrollbar_position.connect(
+                self.restore_scrollbar_position)
+            self.sig_pythonpath_changed.connect(self.main.pythonpath_changed)
+
+        self.register_project_type(self, EmptyProject)
+        self.setup()
+
+    @on_plugin_available(plugin=Plugins.Editor)
+    def on_editor_available(self):
+        self.editor = self.get_plugin(Plugins.Editor)
+        widget = self.get_widget()
+        treewidget = widget.treewidget
+
+        treewidget.sig_open_file_requested.connect(self.editor.load)
+        treewidget.sig_removed.connect(self.editor.removed)
+        treewidget.sig_tree_removed.connect(self.editor.removed_tree)
+        treewidget.sig_renamed.connect(self.editor.renamed)
+        treewidget.sig_tree_renamed.connect(self.editor.renamed_tree)
+        treewidget.sig_module_created.connect(self.editor.new)
+        treewidget.sig_file_created.connect(
+            lambda t: self.editor.new(text=t))
+
+        self.sig_project_loaded.connect(
+            lambda v: self.editor.setup_open_files())
+        self.sig_project_closed[bool].connect(
+            lambda v: self.editor.setup_open_files())
+        self.editor.set_projects(self)
+        self.sig_project_loaded.connect(
+            lambda v: self.editor.set_current_project_path(v))
+        self.sig_project_closed.connect(
+            lambda v: self.editor.set_current_project_path())
+
+    @on_plugin_available(plugin=Plugins.Completions)
+    def on_completions_available(self):
+        self.completions = self.get_plugin(Plugins.Completions)
 
         # TODO: This is not necessary anymore due to us starting workspace
         # services in the editor. However, we could restore it in the future.
         # completions.sig_language_completions_available.connect(
         #     lambda settings, language:
         #         self.start_workspace_services())
-        if completions:
-            completions.sig_stop_completions.connect(
-                self.stop_workspace_services)
-
-        # New project connections. Order matters!
+        self.completions.sig_stop_completions.connect(
+            self.stop_workspace_services)
         self.sig_project_loaded.connect(
-            lambda path:
-            self.main.workingdirectory.chdir(
-                directory=path,
-                sender_plugin=self
-            )
-        )
-        self.sig_project_loaded.connect(
-            lambda v: self.main.set_window_title())
-
-        if completions:
-            self.sig_project_loaded.connect(
-                functools.partial(completions.project_path_update,
-                                  update_kind=WorkspaceUpdateKind.ADDITION,
-                                  instance=self))
-        self.sig_project_loaded.connect(
-            lambda v: self.main.editor.setup_open_files())
-        self.sig_project_loaded.connect(self.update_explorer)
-        if outlineexplorer:
-            self.sig_project_loaded.connect(
-                lambda v: self.main.outlineexplorer.update_all_editors())
-        self.sig_project_closed[object].connect(
-            lambda path:
-            self.main.workingdirectory.chdir(
-                directory=self.get_last_working_dir(),
-                sender_plugin=self
-            )
-        )
+            functools.partial(self.completions.project_path_update,
+                              update_kind=WorkspaceUpdateKind.ADDITION,
+                              instance=self))
         self.sig_project_closed.connect(
-            lambda v: self.main.set_window_title())
-        if completions:
-            self.sig_project_closed.connect(
-                functools.partial(completions.project_path_update,
-                                  update_kind=WorkspaceUpdateKind.DELETION,
-                                  instance=self))
-        self.sig_project_closed.connect(
-            lambda v: self.main.editor.setup_open_files())
-        if outlineexplorer:
-            self.sig_project_closed.connect(
-                lambda v: self.main.outlineexplorer.update_all_editors())
+            functools.partial(self.completions.project_path_update,
+                              update_kind=WorkspaceUpdateKind.DELETION,
+                              instance=self))
+
+    @on_plugin_available(plugin=Plugins.IPythonConsole)
+    def on_ipython_console_available(self):
+        self.ipyconsole = self.get_plugin(Plugins.IPythonConsole)
+        widget = self.get_widget()
+        treewidget = widget.treewidget
+
+        treewidget.sig_open_interpreter_requested.connect(
+            self.ipyconsole.create_client_from_path)
+        treewidget.sig_run_requested.connect(
+            lambda fname:
+            self.ipyconsole.run_script(
+                fname, osp.dirname(fname), '', False, False, False, True,
+                False)
+        )
+
+    @on_plugin_available(plugin=Plugins.MainMenu)
+    def on_main_menu_available(self):
+        main_menu = self.get_plugin(Plugins.MainMenu)
+        new_project_action = self.get_action(ProjectsActions.NewProject)
+        open_project_action = self.get_action(ProjectsActions.OpenProject)
+
+        projects_menu = main_menu.get_application_menu(
+            ApplicationMenus.Projects)
+        projects_menu.aboutToShow.connect(self.is_invalid_active_project)
+
+        main_menu.add_item_to_application_menu(
+            new_project_action,
+            menu_id=ApplicationMenus.Projects,
+            section=ProjectsMenuSections.New)
+
+        for item in [open_project_action, self.close_project_action,
+                     self.delete_project_action]:
+            main_menu.add_item_to_application_menu(
+                item,
+                menu_id=ApplicationMenus.Projects,
+                section=ProjectsMenuSections.Open)
+
+        main_menu.add_item_to_application_menu(
+            self.recent_project_menu,
+            menu_id=ApplicationMenus.Projects,
+            section=ProjectsMenuSections.Extras)
+
+    def setup(self):
+        """Setup the plugin actions."""
+        self.create_action(
+            ProjectsActions.NewProject,
+            text=_("New Project..."),
+            triggered=self.create_new_project)
+
+        self.create_action(
+            ProjectsActions.OpenProject,
+            text=_("Open Project..."),
+            triggered=lambda v: self.open_project())
+
+        self.close_project_action = self.create_action(
+            ProjectsActions.CloseProject,
+            text=_("Close Project"),
+            triggered=self.close_project)
+
+        self.delete_project_action = self.create_action(
+            ProjectsActions.DeleteProject,
+            text=_("Delete Project"),
+            triggered=self.delete_project)
+
+        self.clear_recent_projects_action = self.create_action(
+            ProjectsActions.ClearRecentProjects,
+            text=_("Clear this list"),
+            triggered=self.clear_recent_projects)
+
+        self.max_recent_action = self.create_action(
+            ProjectsActions.MaxRecent,
+            text=_("Maximum number of recent projects..."),
+            triggered=self.change_max_recent_projects)
+
+        self.recent_project_menu = self.get_widget().create_menu(
+            ProjectsMenuSubmenus.RecentProjects,
+            _("Recent Projects")
+        )
         self.recent_project_menu.aboutToShow.connect(self.setup_menu_actions)
-
-        self.main.restore_scrollbar_position.connect(
-                                               self.restore_scrollbar_position)
-        self.sig_pythonpath_changed.connect(self.main.pythonpath_changed)
-        self.main.editor.set_projects(self)
-
-        self.sig_project_loaded.connect(
-            lambda v: self.main.editor.set_current_project_path(v))
-        self.sig_project_closed.connect(
-            lambda v: self.main.editor.set_current_project_path())
-
-        # Connect to file explorer to keep single click to open files in sync
-        # TODO: Remove this once projects is migrated
-        CONF.observe_configuration(self, 'explorer', 'single_click_to_open')
-        self.register_project_type(self, EmptyProject)
-
-    def on_configuration_change(self, option, section, value):
-        """Set single click to open files and directories."""
-        if option == 'single_click_to_open':
-            self.explorer.treewidget.set_single_click_to_open(value)
-
-    def closing_plugin(self, cancelable=False):
-        """Perform actions before parent main window is closed"""
-        self.save_config()
-        self.explorer.closing_widget()
-        return True
-
-    def unmaximize(self):
-        """Unmaximize the currently maximized plugin, if not self."""
-        if (self.main.last_plugin is not None and
-                self.main.last_plugin._ismaximized and
-                self.main.last_plugin is not self):
-            self.main.maximize_dockwidget()
-
-    def build_opener(self, project):
-        """Build function opening passed project"""
-        def opener(*args, **kwargs):
-            self.open_project(path=project)
-        return opener
-
-    # ------ Public API -------------------------------------------------------
-    def on_first_registration(self):
-        """Action to be performed on first plugin registration"""
-        # TODO: Uncomment for Spyder 5
-        # self.tabify(self.main.explorer)
+        self.setup_menu_actions()
 
     def setup_menu_actions(self):
         """Setup and update the menu actions."""
-        self.recent_project_menu.clear()
-        self.recent_projects_actions = []
         if self.recent_projects:
             for project in self.recent_projects:
                 if self.is_valid_project(project):
-                    name = project.replace(get_home_dir(), '~')
-                    action = create_action(
-                        self,
-                        name,
-                        icon=ima.icon('project'),
-                        triggered=self.build_opener(project),
-                    )
-                    self.recent_projects_actions.append(action)
-                else:
-                    self.recent_projects.remove(project)
-            self.recent_projects_actions += [
-                None,
-                self.clear_recent_projects_action,
-                self.max_recent_action
-            ]
-        else:
-            self.recent_projects_actions = [self.clear_recent_projects_action,
-                                            self.max_recent_action]
-        add_actions(self.recent_project_menu, self.recent_projects_actions)
+                    if os.name == 'nt':
+                        name = project
+                    else:
+                        name = project.replace(get_home_dir(), '~')
+                    try:
+                        action = self.get_action(name)
+                    except KeyError:
+                        action = self.create_action(
+                            name,
+                            text=name,
+                            icon=ima.icon('project'),
+                            triggered=self.build_opener(project),
+                        )
+                    self.get_widget().add_item_to_menu(
+                        action,
+                        menu=self.recent_project_menu,
+                        section=RecentProjectsMenuSections.Recent)
+
+        for item in [self.clear_recent_projects_action,
+                     self.max_recent_action]:
+            self.get_widget().add_item_to_menu(
+                item,
+                menu=self.recent_project_menu,
+                section=RecentProjectsMenuSections.Extras)
         self.update_project_actions()
 
     def update_project_actions(self):
@@ -322,31 +344,39 @@ class Projects(SpyderPluginWidget):
         self.close_project_action.setEnabled(active)
         self.delete_project_action.setEnabled(active)
 
+    def on_close(self, cancelable=False):
+        """Perform actions before parent main window is closed"""
+        self.save_config()
+        return True
+
+    def unmaximize(self):
+        """Unmaximize the currently maximized plugin, if not self."""
+        if self.main:
+            if (self.main.last_plugin is not None and
+                    self.main.last_plugin._ismaximized and
+                    self.main.last_plugin is not self):
+                self.main.maximize_dockwidget()
+
+    def build_opener(self, project):
+        """Build function opening passed project"""
+        def opener(*args, **kwargs):
+            self.open_project(path=project)
+        return opener
+
+    # ------ Public API -------------------------------------------------------
     @Slot()
     def create_new_project(self):
         """Create new project."""
         self.unmaximize()
-        active_project = self.current_active_project
-        dlg = ProjectDialog(self, project_types=self.get_project_types())
+        dlg = ProjectDialog(self.get_widget(),
+                            project_types=self.get_project_types())
         result = dlg.exec_()
         data = dlg.project_data
         root_path = data.get("root_path", None)
         project_type = data.get("project_type", EmptyProject.ID)
 
         if result:
-            # A project was not open before
-            if active_project is None:
-                if self.get_option('visible_if_project_open'):
-                    self.show_explorer()
-            else:
-                # We are switching projects.
-                # TODO: Don't emit sig_project_closed when we support
-                # multiple workspaces.
-                self.sig_project_closed.emit(active_project.root_path)
-
             self._create_project(root_path, project_type_id=project_type)
-            self.sig_pythonpath_changed.emit()
-            self.restart_consoles()
             dlg.close()
 
     def _create_project(self, root_path, project_type_id=EmptyProject.ID,
@@ -362,7 +392,8 @@ class Projects(SpyderPluginWidget):
 
             created_succesfully, message = project.create_project()
             if not created_succesfully:
-                QMessageBox.warning(self, "Project creation", message)
+                QMessageBox.warning(
+                    self.get_widget(), "Project creation", message)
                 shutil.rmtree(root_path, ignore_errors=True)
                 return
 
@@ -372,7 +403,7 @@ class Projects(SpyderPluginWidget):
         else:
             if not running_under_pytest():
                 QMessageBox.critical(
-                    self,
+                    self.get_widget(),
                     _('Error'),
                     _("<b>{}</b> is not a registered Spyder project "
                       "type!").format(project_type_id)
@@ -384,14 +415,14 @@ class Projects(SpyderPluginWidget):
         self.unmaximize()
         if path is None:
             basedir = get_home_dir()
-            path = getexistingdirectory(parent=self,
+            path = getexistingdirectory(parent=self.get_widget(),
                                         caption=_("Open project"),
                                         basedir=basedir)
             path = encoding.to_unicode_from_fs(path)
             if not self.is_valid_project(path):
                 if path:
                     QMessageBox.critical(
-                        self,
+                        self.get_widget(),
                         _('Error'),
                         _("<b>%s</b> is not a Spyder project!") % path,
                     )
@@ -407,20 +438,19 @@ class Projects(SpyderPluginWidget):
 
         # A project was not open before
         if self.current_active_project is None:
-            if save_previous_files and self.main.editor is not None:
-                self.main.editor.save_open_files()
+            if save_previous_files and self.editor is not None:
+                self.editor.save_open_files()
 
-            if self.main.editor is not None:
-                self.main.editor.set_option('last_working_dir',
-                                            getcwd_or_home())
+            if self.editor is not None:
+                self.set_conf('last_working_dir', getcwd_or_home(),
+                              section='editor')
 
-            if self.get_option('visible_if_project_open'):
+            if self.get_conf('visible_if_project_open'):
                 self.show_explorer()
         else:
             # We are switching projects
-            if self.main.editor is not None:
-                self.set_project_filenames(
-                    self.main.editor.get_open_filenames())
+            if self.editor is not None:
+                self.set_project_filenames(self.editor.get_open_filenames())
 
             # TODO: Don't emit sig_project_closed when we support
             # multiple workspaces.
@@ -432,7 +462,7 @@ class Projects(SpyderPluginWidget):
         self.latest_project = project
         self.add_to_recent(path)
 
-        self.set_option('current_project_path', self.get_active_project_path())
+        self.set_conf('current_project_path', self.get_active_project_path())
 
         self.setup_menu_actions()
         if workdir and osp.isdir(workdir):
@@ -447,7 +477,7 @@ class Projects(SpyderPluginWidget):
 
         open_successfully, message = project.open_project()
         if not open_successfully:
-            QMessageBox.warning(self, "Project open", message)
+            QMessageBox.warning(self.get_widget(), "Project open", message)
 
     def close_project(self):
         """
@@ -456,28 +486,30 @@ class Projects(SpyderPluginWidget):
         """
         if self.current_active_project:
             self.unmaximize()
-            if self.main.editor is not None:
+            if self.editor is not None:
                 self.set_project_filenames(
-                    self.main.editor.get_open_filenames())
+                    self.editor.get_open_filenames())
             path = self.current_active_project.root_path
             closed_sucessfully, message = (
                 self.current_active_project.close_project())
             if not closed_sucessfully:
-                QMessageBox.warning(self, "Project close", message)
+                QMessageBox.warning(
+                    self.get_widget(), "Project close", message)
 
             self.current_active_project = None
-            self.set_option('current_project_path', None)
+            self.set_conf('current_project_path', None)
             self.setup_menu_actions()
 
             self.sig_project_closed.emit(path)
+            self.sig_project_closed[bool].emit(True)
             self.sig_pythonpath_changed.emit()
 
-            if self.dockwidget is not None:
-                self.set_option('visible_if_project_open',
-                                self.dockwidget.isVisible())
-                self.dockwidget.close()
+            # Hide pane.
+            self.set_conf('visible_if_project_open',
+                          self.get_widget().isVisible())
+            self.toggle_view(False)
 
-            self.explorer.clear()
+            self.get_widget().clear()
             self.restart_consoles()
             self.watcher.stop()
 
@@ -490,7 +522,7 @@ class Projects(SpyderPluginWidget):
             path = self.current_active_project.root_path
             buttons = QMessageBox.Yes | QMessageBox.No
             answer = QMessageBox.warning(
-                self,
+                self.get_widget(),
                 _("Delete"),
                 _("Do you really want to delete <b>{filename}</b>?<br><br>"
                   "<b>Note:</b> This action will only delete the project. "
@@ -503,7 +535,7 @@ class Projects(SpyderPluginWidget):
                     shutil.rmtree(osp.join(path, '.spyproject'))
                 except EnvironmentError as error:
                     QMessageBox.critical(
-                        self,
+                        self.get_widget(),
                         _("Project Explorer"),
                         _("<b>Unable to delete <i>{varpath}</i></b>"
                           "<br><br>The error message was:<br>{error}"
@@ -512,21 +544,22 @@ class Projects(SpyderPluginWidget):
     def clear_recent_projects(self):
         """Clear the list of recent projects"""
         self.recent_projects = []
+        self.set_conf('recent_projects', self.recent_projects)
         self.setup_menu_actions()
 
     def change_max_recent_projects(self):
         """Change max recent projects entries."""
 
         mrf, valid = QInputDialog.getInt(
-            self,
+            self.get_widget(),
             _('Projects'),
             _('Maximum number of recent projects'),
-            self.get_option('max_recent_projects'),
+            self.get_conf('max_recent_projects'),
             1,
             35)
 
         if valid:
-            self.set_option('max_recent_projects', mrf)
+            self.set_conf('max_recent_projects', mrf)
 
     def get_active_project(self):
         """Get the active project"""
@@ -536,8 +569,8 @@ class Projects(SpyderPluginWidget):
         """
         Reopen the active project when Spyder was closed last time, if any
         """
-        current_project_path = self.get_option('current_project_path',
-                                               default=None)
+        current_project_path = self.get_conf('current_project_path',
+                                             default=None)
 
         # Needs a safer test of project existence!
         if (current_project_path and
@@ -573,8 +606,8 @@ class Projects(SpyderPluginWidget):
     def get_pythonpath(self, at_start=False):
         """Get project path as a list to be added to PYTHONPATH"""
         if at_start:
-            current_path = self.get_option('current_project_path',
-                                           default=None)
+            current_path = self.get_conf('current_project_path',
+                                         default=None)
         else:
             current_path = self.get_active_project_path()
         if current_path is None:
@@ -584,8 +617,8 @@ class Projects(SpyderPluginWidget):
 
     def get_last_working_dir(self):
         """Get the path of the last working directory"""
-        return self.main.editor.get_option('last_working_dir',
-                                           default=getcwd_or_home())
+        return self.get_conf(
+            'last_working_dir', section='editor', default=getcwd_or_home())
 
     def save_config(self):
         """
@@ -593,54 +626,73 @@ class Projects(SpyderPluginWidget):
 
         Also save whether dock widget is visible if a project is open.
         """
-        self.set_option('recent_projects', self.recent_projects)
-        self.set_option('expanded_state',
-                        self.explorer.treewidget.get_expanded_state())
-        self.set_option('scrollbar_position',
-                        self.explorer.treewidget.get_scrollbar_position())
-        if self.current_active_project and self.dockwidget:
-            self.set_option('visible_if_project_open',
-                            self.dockwidget.isVisible())
+        self.set_conf('recent_projects', self.recent_projects)
+        self.set_conf('expanded_state',
+                      self.get_widget().treewidget.get_expanded_state())
+        self.set_conf('scrollbar_position',
+                      self.get_widget().treewidget.get_scrollbar_position())
+        if self.current_active_project:
+            self.set_conf('visible_if_project_open',
+                          self.get_widget().isVisible())
 
     def load_config(self):
         """Load configuration: opened projects & tree widget state"""
-        expanded_state = self.get_option('expanded_state', None)
+        expanded_state = self.get_conf('expanded_state', None)
         # Sometimes the expanded state option may be truncated in .ini file
         # (for an unknown reason), in this case it would be converted to a
         # string by 'userconfig':
         if is_text_string(expanded_state):
             expanded_state = None
         if expanded_state is not None:
-            self.explorer.treewidget.set_expanded_state(expanded_state)
+            self.get_widget().treewidget.set_expanded_state(expanded_state)
 
     def restore_scrollbar_position(self):
         """Restoring scrollbar position after main window is visible"""
-        scrollbar_pos = self.get_option('scrollbar_position', None)
+        scrollbar_pos = self.get_conf('scrollbar_position', None)
         if scrollbar_pos is not None:
-            self.explorer.treewidget.set_scrollbar_position(scrollbar_pos)
+            self.get_widget().treewidget.set_scrollbar_position(scrollbar_pos)
 
     def update_explorer(self):
         """Update explorer tree"""
-        self.explorer.setup_project(self.get_active_project_path())
+        self.get_widget().setup_project(self.get_active_project_path())
 
     def show_explorer(self):
         """Show the explorer"""
-        if self.dockwidget is not None:
-            if self.dockwidget.isHidden():
-                self.dockwidget.show()
-            self.dockwidget.raise_()
-            self.dockwidget.update()
+        if self.get_widget() is not None:
+            self.toggle_view(True)
+            self.get_widget().setVisible(True)
+            self.get_widget().raise_()
+            self.get_widget().update()
 
     def restart_consoles(self):
         """Restart consoles when closing, opening and switching projects"""
-        if self.main.ipyconsole is not None:
-            self.main.ipyconsole.restart()
+        if self.ipyconsole is not None:
+            self.ipyconsole.restart()
 
     def is_valid_project(self, path):
         """Check if a directory is a valid Spyder project"""
         spy_project_dir = osp.join(path, '.spyproject')
         return osp.isdir(path) and osp.isdir(spy_project_dir)
 
+    def is_invalid_active_project(self):
+        """Handle an invalid active project."""
+        try:
+            path = self.get_active_project_path()
+        except AttributeError:
+            return
+
+        if bool(path):
+            if not self.is_valid_project(path):
+                if path:
+                    QMessageBox.critical(
+                        self.get_widget(),
+                        _('Error'),
+                        _("<b>{}</b> is no longer a valid Spyder project! "
+                          "Since it is the current active project, it will "
+                          "be closed automatically.").format(path)
+                    )
+                self.close_project()
+
     def add_to_recent(self, project):
         """
         Add an entry to recent projetcs
@@ -649,7 +701,7 @@ class Projects(SpyderPluginWidget):
         """
         if project not in self.recent_projects:
             self.recent_projects.insert(0, project)
-        if len(self.recent_projects) > self.get_option('max_recent_projects'):
+        if len(self.recent_projects) > self.get_conf('max_recent_projects'):
             self.recent_projects.pop(-1)
 
     def start_workspace_services(self):
@@ -665,11 +717,10 @@ class Projects(SpyderPluginWidget):
 
     def emit_request(self, method, params, requires_response):
         """Send request/notification/response to all LSP servers."""
-        completions = self.main.get_plugin(Plugins.Completions, error=False)
-        if completions:
-            params['requires_response'] = requires_response
-            params['response_instance'] = self
-            self.main.completions.broadcast_notification(method, params)
+        params['requires_response'] = requires_response
+        params['response_instance'] = self
+        if self.completions:
+            self.completions.broadcast_notification(method, params)
 
     @Slot(str, dict)
     def handle_response(self, method, params):
@@ -814,7 +865,6 @@ class Projects(SpyderPluginWidget):
             project_type_id = config[WORKSPACE].get(
                 "project_type", EmptyProject.ID)
 
-
         EmptyProject._PARENT_PLUGIN = self
         project_types = self.get_project_types()
         project_type_class = project_types.get(project_type_id, EmptyProject)
@@ -854,12 +904,3 @@ class Projects(SpyderPluginWidget):
             are project type classes.
         """
         return self._project_types
-
-    # TODO: To be removed after migration
-    def get_plugin(self, plugin_name):
-        """
-        Return a plugin instance by providing the plugin's NAME.
-        """
-        PLUGINS = self.main._PLUGINS
-        if plugin_name in PLUGINS:
-            return PLUGINS[plugin_name]
